use std::collections::BTreeMap;

use bevy::{math::VectorSpace, picking::pointer::PointerId, prelude::*, utils::HashMap};

use crate::map::{self, MapMarker};

pub struct Plugin;
impl bevy::prelude::Plugin for Plugin {
    fn build(&self, app: &mut App) {
        app.add_systems(Startup, setup);
    }
}

#[derive(Component)]
struct PointersDragging(HashMap<PointerId, Vec2>);

#[derive(Component)]
pub struct MapUIComponent;

fn setup(
    mut cmds: Commands
) {
    cmds.spawn((
        Node {
            width: Val::Percent(100.0),
            height: Val::Percent(100.0),
            align_items: AlignItems::Center,
            justify_content: JustifyContent::Center,
            ..default()
        },
        PickingBehavior {
            should_block_lower: false,
            is_hoverable: true
        },
        MapUIComponent,
        PointersDragging(HashMap::new())
    )).observe(|trigger: Trigger<Pointer<DragStart>>, mut ptrs: Query<&mut PointersDragging>| {
        let event = trigger.event();
        // dbg!(event);
        if event.button == PointerButton::Primary {
            ptrs.get_mut(event.target).unwrap().0.insert(event.pointer_id, event.pointer_location.position);
        }
    }).observe(|trigger: Trigger<Pointer<DragEnd>>, mut ptrs: Query<&mut PointersDragging>| {
        let event = trigger.event();
        // dbg!(event);
        if event.button == PointerButton::Primary {
            ptrs.get_mut(event.target).unwrap().0.remove(&event.pointer_id);
        }
    }).observe(|
            trigger: Trigger<Pointer<Drag>>,
            mut ptrs: Query<&mut PointersDragging>,
            mut cam: Query<&mut Transform, With<Camera2d>>,
        | {
            let event = trigger.event();
            // dbg!(event);
            if event.button == PointerButton::Primary {
                let mut cam = cam.single_mut();
                let mut ptrs = ptrs.get_mut(event.target).unwrap();
                
                let old_midpoint = ptrs.0.values().fold(Vec2::ZERO, |acc, pos| acc + (pos/ptrs.0.len() as f32));
                ptrs.0.insert(event.pointer_id, event.pointer_location.position);
                let new_midpoint = ptrs.0.values().fold(Vec2::ZERO, |acc, pos| acc + (pos/ptrs.0.len() as f32));

                cam.translation.x -= (new_midpoint.x - old_midpoint.x)*cam.scale.x;
                cam.translation.y += (new_midpoint.y - old_midpoint.y)*cam.scale.y;
            }
        }
    );
    // Spawn all ui elements as children of this one
}